from __future__ import annotations

import json
import subprocess
import uuid
from pathlib import Path
from types import SimpleNamespace
from typing import Any

import httpx

from .config import DEFAULT_MODEL


def _completion_api_key(cfg: dict[str, Any]) -> str:
    api_key = str(cfg.get("api_key") or "").strip()
    if api_key.startswith("os.environ/"):
        env_name = api_key.partition("/")[2].strip()
        if env_name:
            try:
                import os

                return str(os.getenv(env_name) or "").strip()
            except Exception:
                return ""
    if api_key:
        return api_key
    env_name = str(cfg.get("api_key_env") or "").strip()
    if env_name:
        try:
            import os

            return str(os.getenv(env_name) or "").strip()
        except Exception:
            return ""
    return ""


def _normalized_reasoning_effort(cfg: dict[str, Any]) -> str:
    value = str(cfg.get("reasoning_effort") or "").strip().lower()
    return value if value in ("minimal", "low", "medium", "high", "xhigh", "none") else ""


def should_use_codex_mirror_transport(cfg: dict[str, Any]) -> bool:
    provider_name = str(cfg.get("model_provider") or "").strip().lower()
    api_base = str(cfg.get("api_base") or "").strip().lower()
    custom_llm_provider = str(cfg.get("custom_llm_provider") or "").strip().lower()
    wire_api = str(cfg.get("wire_api") or "").strip().lower()
    if provider_name == "mirror":
        return True
    return bool(api_base.endswith("/codex") and custom_llm_provider == "openai" and wire_api == "responses")


def _workspace_root() -> Path:
    return Path.cwd().resolve()


def _git_metadata(root: Path) -> dict[str, Any]:
    metadata: dict[str, Any] = {"has_changes": False}
    try:
        origin = subprocess.check_output(
            ["git", "config", "--get", "remote.origin.url"],
            text=True,
            cwd=str(root),
            stderr=subprocess.DEVNULL,
        ).strip()
        if origin:
            metadata["associated_remote_urls"] = {"origin": origin}
    except Exception:
        pass
    try:
        commit = subprocess.check_output(
            ["git", "rev-parse", "HEAD"],
            text=True,
            cwd=str(root),
            stderr=subprocess.DEVNULL,
        ).strip()
        if commit:
            metadata["latest_git_commit_hash"] = commit
    except Exception:
        pass
    try:
        status = subprocess.check_output(
            ["git", "status", "--porcelain"],
            text=True,
            cwd=str(root),
            stderr=subprocess.DEVNULL,
        ).strip()
        metadata["has_changes"] = bool(status)
    except Exception:
        pass
    return metadata


def _request_headers(*, cfg: dict[str, Any], session_id: str | None = None) -> tuple[str, dict[str, str]]:
    del cfg
    request_id = str(session_id or uuid.uuid4())
    root = _workspace_root()
    metadata = {
        "turn_id": str(uuid.uuid4()),
        "workspaces": {
            str(root): _git_metadata(root),
        },
        "sandbox": "seatbelt",
    }
    headers = {
        "accept": "text/event-stream",
        "content-type": "application/json",
        "user-agent": "Codex Desktop/0.115.0 (Mac OS 26.3.1; arm64) dumb (codex-exec; 0.115.0)",
        "originator": "Codex Desktop",
        "x-client-request-id": request_id,
        "session_id": request_id,
        "x-codex-turn-metadata": json.dumps(metadata, ensure_ascii=False, separators=(",", ":")),
    }
    return request_id, headers


def _convert_content(content: Any, role: str) -> list[dict[str, Any]]:
    if content is None:
        return []
    if isinstance(content, str):
        text_type = "output_text" if role == "assistant" else "input_text"
        return [{"type": text_type, "text": content}]
    if isinstance(content, list):
        items: list[dict[str, Any]] = []
        for item in content:
            if isinstance(item, str):
                items.extend(_convert_content(item, role))
                continue
            if not isinstance(item, dict):
                items.extend(_convert_content(str(item), role))
                continue
            item_type = str(item.get("type") or "").strip()
            if item_type == "text":
                items.extend(_convert_content(str(item.get("text") or ""), role))
                continue
            if item_type == "image_url":
                image_url = item.get("image_url")
                if isinstance(image_url, dict):
                    url = str(image_url.get("url") or "").strip()
                    detail = str(image_url.get("detail") or "").strip()
                else:
                    url = str(image_url or "").strip()
                    detail = ""
                if url:
                    payload: dict[str, Any] = {"type": "input_image", "image_url": url}
                    if detail:
                        payload["detail"] = detail
                    items.append(payload)
                continue
            text_value = str(item.get("text") or "").strip()
            if text_value:
                items.extend(_convert_content(text_value, role))
        return items
    return _convert_content(str(content), role)


def _convert_tools(tools: Any) -> list[dict[str, Any]]:
    if not isinstance(tools, list):
        return []
    converted: list[dict[str, Any]] = []
    for tool in tools:
        if not isinstance(tool, dict) or str(tool.get("type") or "").strip() != "function":
            continue
        function = tool.get("function")
        if not isinstance(function, dict):
            continue
        name = str(function.get("name") or "").strip()
        if not name:
            continue
        converted_tool: dict[str, Any] = {
            "type": "function",
            "name": name,
            "parameters": function.get("parameters") or {},
        }
        description = str(function.get("description") or "").strip()
        if description:
            converted_tool["description"] = description
        if function.get("strict") is not None:
            converted_tool["strict"] = bool(function.get("strict"))
        converted.append(converted_tool)
    return converted


def _messages_to_input(messages: list[dict[str, Any]]) -> tuple[list[dict[str, Any]], str]:
    input_items: list[dict[str, Any]] = []
    instructions: list[str] = []
    for msg in messages:
        if not isinstance(msg, dict):
            continue
        role = str(msg.get("role") or "").strip()
        content = msg.get("content")
        tool_calls = msg.get("tool_calls")
        if role == "system":
            if isinstance(content, str):
                text = content.strip()
                if text:
                    instructions.append(text)
            else:
                parts = _convert_content(content, "system")
                if parts:
                    input_items.append({"type": "message", "role": "system", "content": parts})
            continue
        if role == "tool":
            output_parts = _convert_content(content, "user")
            input_items.append(
                {
                    "type": "function_call_output",
                    "call_id": str(msg.get("tool_call_id") or "").strip(),
                    "output": output_parts,
                }
            )
            continue
        if role == "assistant" and isinstance(tool_calls, list) and tool_calls:
            for tool_call in tool_calls:
                if not isinstance(tool_call, dict):
                    continue
                function = tool_call.get("function")
                if not isinstance(function, dict):
                    continue
                input_items.append(
                    {
                        "type": "function_call",
                        "call_id": str(tool_call.get("id") or "").strip(),
                        "name": str(function.get("name") or "").strip(),
                        "arguments": str(function.get("arguments") or ""),
                    }
                )
            continue
        input_items.append(
            {
                "type": "message",
                "role": role or "user",
                "content": _convert_content(content, role or "user"),
            }
        )
    return input_items, "\n\n".join(part for part in instructions if part).strip()


def build_request_body(
    *,
    cfg: dict[str, Any],
    messages: list[dict[str, Any]],
    tools: list[dict[str, Any]],
    session_id: str,
    stream: bool,
) -> dict[str, Any]:
    input_items, instructions = _messages_to_input(messages)
    body: dict[str, Any] = {
        "model": str(cfg.get("model") or DEFAULT_MODEL).strip() or DEFAULT_MODEL,
        "input": input_items,
        "tools": _convert_tools(tools),
        "tool_choice": "auto",
        "parallel_tool_calls": True,
        "store": False,
        "stream": stream,
        "include": ["reasoning.encrypted_content"],
        "prompt_cache_key": session_id,
        "text": {"verbosity": "low"},
    }
    if instructions:
        body["instructions"] = instructions
    effort = _normalized_reasoning_effort(cfg)
    if effort:
        body["reasoning"] = {"effort": effort}
    return body


def _iter_sse_events(response: httpx.Response):
    event_name = ""
    data_parts: list[str] = []
    for raw_line in response.iter_lines():
        line = str(raw_line or "")
        if not line:
            if data_parts:
                yield event_name or "message", "\n".join(data_parts)
            event_name = ""
            data_parts = []
            continue
        if line.startswith("event:"):
            event_name = line.partition(":")[2].strip()
            continue
        if line.startswith("data:"):
            data_parts.append(line.partition(":")[2].strip())
    if data_parts:
        yield event_name or "message", "\n".join(data_parts)


def _error_text(response: httpx.Response) -> str:
    try:
        text = response.read().decode(errors="replace").strip()
    except Exception:
        text = ""
    if text:
        return text
    return f"http {response.status_code}"


def stream_response(
    *,
    cfg: dict[str, Any],
    messages: list[dict[str, Any]],
    tools: list[dict[str, Any]],
    on_text_delta: Any | None = None,
) -> dict[str, Any]:
    request_id, headers = _request_headers(cfg=cfg)
    api_key = _completion_api_key(cfg)
    if not api_key:
        raise RuntimeError("mirror provider is missing api_key")
    headers["authorization"] = f"Bearer {api_key}"
    body = build_request_body(
        cfg=cfg,
        messages=messages,
        tools=tools,
        session_id=request_id,
        stream=True,
    )
    url = str(cfg.get("api_base") or "").rstrip("/") + "/responses"
    with httpx.stream("POST", url, headers=headers, json=body, timeout=120) as response:
        final_response: dict[str, Any] | None = None
        if response.status_code >= 400:
            raise RuntimeError(_error_text(response))
        content_type = str(response.headers.get("content-type") or "").lower()
        if "text/event-stream" not in content_type:
            raise RuntimeError(_error_text(response))
        for _, data_text in _iter_sse_events(response):
            try:
                payload = json.loads(data_text)
            except Exception:
                continue
            if not isinstance(payload, dict):
                continue
            payload_type = str(payload.get("type") or "").strip()
            if payload_type == "response.output_text.delta":
                delta = str(payload.get("delta") or "")
                if delta and callable(on_text_delta):
                    on_text_delta(delta)
                continue
            if payload_type == "response.completed":
                final_response = dict(payload.get("response") or {})
        if not isinstance(final_response, dict) or not final_response:
            raise RuntimeError("mirror provider returned no completed response")
        return final_response


def message_payload(response_payload: dict[str, Any]) -> dict[str, Any]:
    content_parts: list[dict[str, Any]] = []
    tool_calls: list[dict[str, Any]] = []
    reasoning_details: list[dict[str, Any]] = []
    for item in response_payload.get("output") or []:
        if not isinstance(item, dict):
            continue
        item_type = str(item.get("type") or "").strip()
        if item_type == "message":
            for part in item.get("content") or []:
                if not isinstance(part, dict):
                    continue
                if str(part.get("type") or "").strip() != "output_text":
                    continue
                content_parts.append({"text": str(part.get("text") or "")})
            continue
        if item_type == "function_call":
            tool_calls.append(
                {
                    "id": str(item.get("call_id") or item.get("id") or "").strip(),
                    "type": "function",
                    "function": {
                        "name": str(item.get("name") or "").strip(),
                        "arguments": str(item.get("arguments") or ""),
                    },
                }
            )
            continue
        if item_type == "reasoning":
            reasoning_details.append(dict(item))
    payload: dict[str, Any] = {"content": content_parts}
    if tool_calls:
        payload["tool_calls"] = tool_calls
    if reasoning_details:
        payload["reasoning_details"] = reasoning_details
    return payload


def response_text(response_payload: dict[str, Any]) -> str:
    parts: list[str] = []
    for part in message_payload(response_payload).get("content") or []:
        if isinstance(part, dict):
            text = str(part.get("text") or "")
        else:
            text = str(part or "")
        if text:
            parts.append(text)
    return "\n".join(parts).strip()


def model_response(response_payload: dict[str, Any]) -> Any:
    choice = SimpleNamespace(message=message_payload(response_payload))
    return SimpleNamespace(
        choices=[choice],
        usage=dict(response_payload.get("usage") or {}),
        model=str(response_payload.get("model") or ""),
        response=response_payload,
    )



from __future__ import annotations

import os
import sys
from pathlib import Path


def _usage() -> str:
    return "Usage: entari run [config-file]"


def _default_config_path() -> Path:
    cwd = Path.cwd()
    local = cwd / "entari.yml"
    if local.exists():
        return local

    repo_dev = cwd / "dev.entari" / "entari.yml"
    if repo_dev.exists():
        return repo_dev

    return local


def main() -> int:
    argv = sys.argv[1:]
    if not argv or argv[0] in {"-h", "--help", "help"}:
        print(_usage())
        return 0

    if argv[0] != "run":
        print(_usage(), file=sys.stderr)
        return 2

    config_path = Path(argv[1]).expanduser() if len(argv) > 1 else _default_config_path()
    os.environ.setdefault("ENTARI_CONFIG_FILE", str(config_path))

    from arclet.entari.core import Entari

    app = Entari.load(config_path)
    app.run()
    return 0


if __name__ == "__main__":
    raise SystemExit(main())



from __future__ import annotations

import asyncio
from typing import Any, Dict, List, Optional


def _normalize_region(kl: Optional[str]) -> str:
    raw = str(kl or "").strip().lower()
    return raw or "wt-wt"


def _load_ddgs_class() -> Any:
    try:
        from ddgs import DDGS

        return DDGS
    except Exception as ddgs_exc:
        try:
            from duckduckgo_search import DDGS

            return DDGS
        except Exception:
            raise RuntimeError(
                "ddgs is not installed; reinstall `hyw` or install `ddgs` manually."
            ) from ddgs_exc


def _normalize_search_row(row: Dict[str, Any]) -> Optional[Dict[str, Any]]:
    url = str(
        row.get("href")
        or row.get("url")
        or row.get("link")
        or row.get("source")
        or row.get("document_url")
        or ""
    ).strip()
    if url.startswith("//"):
        url = "https:" + url
    if not url.startswith(("http://", "https://")):
        return None

    title = str(
        row.get("title")
        or row.get("name")
        or row.get("headline")
        or row.get("text")
        or ""
    ).strip() or "No Title"
    snippet = str(
        row.get("body")
        or row.get("snippet")
        or row.get("description")
        or row.get("content")
        or row.get("text")
        or ""
    ).strip()
    if snippet.startswith(title):
        snippet = snippet[len(title) :].strip(" -:\n")

    return {
        "title": title,
        "url": url,
        "snippet": snippet[:500],
        "provider": "ddgs",
    }


async def ddgs_search(
    query: str,
    kl: Optional[str] = None,
    max_results: int = 5,
    **_: Any,
) -> List[Dict[str, Any]]:
    def _run() -> List[Dict[str, Any]]:
        DDGS = _load_ddgs_class()
        kwargs = {
            "region": _normalize_region(kl),
            "safesearch": "moderate",
            "max_results": max(1, int(max_results)),
            "page": 1,
            "backend": "duckduckgo",
        }
        try:
            return list(DDGS().text(query, **kwargs) or [])
        except TypeError:
            kwargs.pop("backend", None)
            return list(DDGS().text(query, **kwargs) or [])

    raw_rows = await asyncio.to_thread(_run)
    results: List[Dict[str, Any]] = []
    seen: set[str] = set()
    for row in raw_rows:
        if not isinstance(row, dict):
            continue
        normalized = _normalize_search_row(row)
        if not normalized:
            continue
        url = normalized["url"]
        if url in seen:
            continue
        seen.add(url)
        results.append(normalized)
        if len(results) >= max(1, int(max_results)):
            break
    return results


__all__ = ["ddgs_search"]



from __future__ import annotations

import asyncio
import base64
import io
from typing import Optional

import httpx
from arclet.entari import Image, MessageChain
from PIL import Image as PILImage


def _compress_image_b64(b64_data: str, quality: int = 85) -> str:
    img_bytes = base64.b64decode(b64_data)
    img = PILImage.open(io.BytesIO(img_bytes))

    if img.mode in ("RGBA", "P"):
        img = img.convert("RGB")

    output = io.BytesIO()
    img.save(output, format="JPEG", quality=quality, optimize=True)
    return base64.b64encode(output.getvalue()).decode("utf-8")


async def _download_image(url: str) -> bytes:
    async with httpx.AsyncClient(timeout=30.0) as client:
        response = await client.get(url)
        response.raise_for_status()
        return response.content


async def process_images(mc: MessageChain, vision_model: Optional[str] = None):
    if vision_model == "off":
        return [], None

    images: list[str] = []
    if mc.get(Image):
        urls = mc[Image].map(lambda item: item.src)
        raw_images = await asyncio.gather(*(_download_image(url) for url in urls))
        for image in raw_images:
            b64_raw = base64.b64encode(image).decode("utf-8")
            images.append(_compress_image_b64(b64_raw, quality=85))

    return images, None


__all__ = ["process_images"]



from __future__ import annotations

import json
import re
from typing import Any
from urllib.parse import urlparse


def sanitize_tool_view_args(value: Any) -> Any:
    if isinstance(value, dict):
        cleaned: dict[str, Any] = {}
        for raw_key, raw_value in value.items():
            key = str(raw_key or "").strip()
            if not key or key.startswith("_"):
                continue
            cleaned[key] = sanitize_tool_view_args(raw_value)
        return cleaned
    if isinstance(value, list):
        return [sanitize_tool_view_args(item) for item in value]
    if isinstance(value, tuple):
        return [sanitize_tool_view_args(item) for item in value]
    return value


def _short_host(url: str) -> str:
    raw_url = str(url or "").strip()
    if not raw_url:
        return ""
    parsed = urlparse(raw_url)
    host = str(parsed.netloc or parsed.path or "").strip()
    if host.startswith("www."):
        host = host[4:]
    return host


def _clean_inline_text(value: Any) -> str:
    return re.sub(r"\s+", " ", str(value or "").strip())


def _truncate_line(text: str, max_chars: int) -> str:
    line = str(text or "").strip()
    if len(line) <= max_chars:
        return line
    return line[: max_chars - 1].rstrip() + "…"


def _window_label(value: Any) -> str:
    text = str(value or "").strip()
    if not text:
        return ""
    if text.lower() == "all":
        return "All"
    match = re.search(r"\d+", text)
    if not match:
        return ""
    return f"L{int(match.group(0))}"


def _numeric_value(*values: Any) -> int | None:
    for value in values:
        if value in (None, ""):
            continue
        try:
            return int(value)
        except Exception:
            continue
    return None


def _list_count(value: Any) -> int:
    return len(value) if isinstance(value, list) else 0


def _plan_counts(payload: dict[str, Any]) -> tuple[int, int]:
    create_count = _numeric_value(payload.get("_created_count"))
    if create_count is None:
        create_count = _list_count(payload.get("create"))
    update_count = _numeric_value(payload.get("_updated_count"))
    if update_count is None:
        update_count = _list_count(payload.get("update"))
    return max(0, int(create_count or 0)), max(0, int(update_count or 0))


def format_tool_view_argument(name: str, arguments: Any, *, max_items: int = 12, max_chars: int = 160) -> str:
    payload = sanitize_tool_view_args(arguments)
    if not isinstance(payload, dict):
        return ""

    if name in ("web_search", "web_search_wiki"):
        query = _clean_inline_text(payload.get("query"))
        return f"\"{query}\"" if query else ""

    if name == "page_extract":
        host = _short_host(str(payload.get("url") or "").strip())
        ref = str(payload.get("ref") or "").strip()
        query = _clean_inline_text(payload.get("query"))
        base = ""
        if query:
            base = f"\"{query}\""
        target = host or ref
        if target:
            base = f"{base} in \"{target}\"".strip()
        extras: list[str] = []
        line_label = _window_label(payload.get("lines"))
        if line_label:
            extras.append(line_label)
        if extras:
            return f"{base} · {' · '.join(extras)}".strip()
        return base

    if name == "context_keep":
        ids = payload.get("ids")
        if isinstance(ids, list) and ids:
            return f"Keep {len(ids)}"
        return ""

    if name == "context_delete":
        ids = payload.get("ids")
        if isinstance(ids, list) and ids:
            return f"Delete {len(ids)}"
        return ""

    if name == "plan_update":
        create_count, update_count = _plan_counts(payload)
        parts: list[str] = []
        if create_count:
            parts.append(f"Create {create_count}")
        if update_count:
            parts.append(f"Update {update_count}")
        return ", ".join(parts)

    if name == "sub_agent_task":
        tools = str(payload.get("tools") or "").strip()
        host = _short_host(str(payload.get("url") or "").strip())
        task = _clean_inline_text(payload.get("task"))
        if task:
            task = task[:120]
        binding = ""
        if tools and host:
            binding = f"{tools}({host})"
        elif tools:
            binding = tools
        elif host:
            binding = host
        if binding and task:
            return f"{binding}, {task}"
        return binding or task

    if not payload:
        return ""
    text = json.dumps(payload, ensure_ascii=False)
    if len(text) > max_chars:
        return text[: max_chars - 1].rstrip() + "…"
    return text


def format_tool_view_text(name: str, arguments: Any, *, max_chars: int = 160) -> str:
    payload = arguments if isinstance(arguments, dict) else {}
    tool_name = str(name or "").strip()

    if tool_name in ("web_search", "web_search_wiki"):
        query = _clean_inline_text(payload.get("query"))
        line = f"Web Search \"{query}\"" if query else "Web Search"
        count = _numeric_value(payload.get("_count"), payload.get("count"))
        if count is not None:
            line += f" · {count}"
        return _truncate_line(line, max_chars)

    if tool_name == "page_extract":
        query = _clean_inline_text(payload.get("query"))
        host = _short_host(str(payload.get("url") or "").strip())
        ref = str(payload.get("ref") or "").strip()
        target = host or (f"#{ref}" if ref else "")
        line = f"Read \"{query}\"" if query else "Read"
        if target:
            line += f" in \"{target}\""
        extras: list[str] = []
        window = _window_label(payload.get("lines") or payload.get("window"))
        if window:
            extras.append(window)
        count = _numeric_value(payload.get("_count"), payload.get("count"))
        if count is not None:
            extras.append(str(count))
        if extras:
            line += " · " + " · ".join(extras)
        return _truncate_line(line, max_chars)

    if tool_name == "context_keep":
        count = _numeric_value(payload.get("_count"), payload.get("count"))
        if count is None:
            ids = payload.get("ids")
            if isinstance(ids, list) and ids:
                count = len(ids)
        if count is not None:
            return f"Context: Keep {count}"
        return "Context: Keep"

    if tool_name == "context_delete":
        count = _numeric_value(payload.get("_count"), payload.get("count"))
        if count is None:
            ids = payload.get("ids")
            if isinstance(ids, list) and ids:
                count = len(ids)
        if count is not None:
            return f"Context: Delete {count}"
        return "Context: Delete"

    if tool_name == "plan_update":
        create_count, update_count = _plan_counts(payload)
        actions: list[str] = []
        if create_count:
            actions.append(f"Create {create_count}")
        if update_count:
            actions.append(f"Update {update_count}")
        if not actions:
            actions.append("Update")
        return _truncate_line("Plan: " + " · ".join(actions), max_chars)

    if tool_name == "sub_agent_task":
        tools = str(payload.get("tools") or "").strip().lower()
        task = _clean_inline_text(payload.get("task"))
        host = _short_host(str(payload.get("url") or "").strip())
        if tools == "websearch" and task:
            return _truncate_line(f"Web Search \"{task}\"", max_chars)
        if tools == "page":
            line = f"Read \"{task}\"" if task else "Read"
            if host:
                line += f" in \"{host}\""
            return _truncate_line(line, max_chars)

    fallback = format_tool_view_argument(tool_name, payload, max_chars=max_chars)
    if fallback:
        return _truncate_line(f"{tool_name} {fallback}", max_chars)
    return _truncate_line(tool_name, max_chars)



"""
core/entari_plugin_hyw.py - entari 插件 (保留原项目绝大部分功能)

功能:
- /q 命令触发问答
- /stop 命令停止该用户最近一次提问
- 引用消息 + 图片处理
- 读取 ~/.hyw/config.yml
- 搜索进度/里程碑推送
- Markdown 图片提取
- 完整的事件捕获与展示
"""
from __future__ import annotations

import asyncio
import base64
import json
import re
import threading
from copy import deepcopy
from dataclasses import dataclass
from datetime import datetime
from typing import Any, Dict, List
from urllib.parse import urlparse

import yaml
from loguru import logger

from arclet.alconna import Alconna, AllParam, Args, Arparma
from arclet.entari import (
    At,
    BasicConfModel,
    Image,
    MessageChain,
    MessageCreatedEvent,
    Quote,
    Session,
    Text,
    command,
    listen,
    metadata,
    plugin_config,
)
from arclet.entari.event.command import CommandReceive
from arclet.entari.event.lifespan import Cleanup, Startup

from .config import DEFAULT_TOOL_SELECTIONS, cfg_get, load_config, normalize_tool_provider_name
from .main import Stats, StopRequestedError, run, run_stream, shutdown_tools, startup_tools
from .render import render_markdown_result
from .tool_view import format_tool_view_argument, format_tool_view_text

try:
    from importlib.metadata import version as get_version

    for _dist_name in ("hyw", "entari-plugin-hyw-ai"):
        try:
            __version__ = get_version(_dist_name)
            break
        except Exception:
            continue
    else:
        __version__ = "6.0.0"
except Exception:
    __version__ = "6.0.0"


_IMG_TAG_RE = re.compile(r"<img[^>]+>", flags=re.IGNORECASE)
_MD_IMAGE_RE = re.compile(r"!\[[^\]]*]\(([^)]+)\)", flags=re.IGNORECASE)
_TASK_LIST_BLOCK_RE = re.compile(r"<task_list\b[^>]*>.*?</task_list>", flags=re.IGNORECASE | re.DOTALL)
_ARTICLE_SKELETON_BLOCK_RE = re.compile(r"<article_skeleton\b[^>]*>.*?</article_skeleton>", flags=re.IGNORECASE | re.DOTALL)
_SEARCH_REWRITE_BLOCK_RE = re.compile(r"<search_rewrite\b[^>]*>.*?</search_rewrite>", flags=re.IGNORECASE | re.DOTALL)
_TOOL_BLOCK_RE = re.compile(
    r"<(?:search|wiki|sub_agent|page|context_keep|plan_update|context_update|context_delete)\b[^>]*>.*?</(?:search|wiki|sub_agent|page|context_keep|plan_update|context_update|context_delete)>",
    flags=re.IGNORECASE | re.DOTALL,
)
_TOOL_SELF_CLOSING_RE = re.compile(
    r"<(?:search|wiki|sub_agent|page|context_keep|plan_update|context_update|context_delete)\b[^>]*/>",
    flags=re.IGNORECASE,
)
_ENTARI_RENDER_PROVIDER = "md2png_lite"


# ── 工具函数 ──────────────────────────────────────────────────

def _as_dict(value: Any) -> dict[str, Any]:
    if isinstance(value, dict):
        return value
    if isinstance(value, str):
        text = str(value or "").strip()
        if not text:
            return {}
        try:
            parsed = json.loads(text)
            return parsed if isinstance(parsed, dict) else {}
        except Exception:
            pass
        try:
            parsed = yaml.safe_load(text)
            return parsed if isinstance(parsed, dict) else {}
        except Exception:
            return {}
    return {}


def _format_web_search_query(arguments: Any) -> str:
    payload = _as_dict(arguments)
    data = payload.get("data") if isinstance(payload.get("data"), dict) else payload
    query_list = data.get("queries") if isinstance(data.get("queries"), list) else []
    if query_list:
        query = " || ".join(str(item or "").strip() for item in query_list if str(item or "").strip())
    else:
        query = str(data.get("query") or data.get("user_message") or data.get("description") or "").strip()
    return query


def _format_search_progress_lines(queries: list[str]) -> str:
    if not queries:
        return ""
    rows = ["🔍 正在搜索:"]
    for i, text in enumerate(queries[:8], start=1):
        rows.append(f"  {i}. {text}")
    return "\n".join(rows).strip()


# ── 插件配置 ──────────────────────────────────────────────────

@dataclass
class PluginConfig(BasicConfModel):
    question_command: str = "/q"
    stop_command: str = "/stop"
    quote: bool = False
    verbose: bool = True
    render_theme: str = "paper"


conf = plugin_config(PluginConfig)


@dataclass
class _ActiveRequest:
    request_id: str
    task: asyncio.Task[Any]
    stop_event: threading.Event


_ACTIVE_REQUESTS: dict[str, list[_ActiveRequest]] = {}
_ACTIVE_REQUESTS_LOCK = threading.Lock()


def _session_request_scope(session: Session[MessageCreatedEvent]) -> str:
    platform = str(getattr(getattr(session, "account", None), "platform", "") or "").strip() or "unknown"
    self_id = str(getattr(getattr(session, "account", None), "self_id", "") or "").strip() or "unknown"
    user_id = ""
    channel_id = ""
    try:
        user_id = str(session.user.id or "").strip()
    except Exception:
        user_id = str(getattr(getattr(session.event, "user", None), "id", "") or "").strip()
    try:
        channel_id = str(session.channel.id or "").strip()
    except Exception:
        channel_id = str(getattr(getattr(session.event, "channel", None), "id", "") or "").strip()
    target = f"user:{user_id}" if user_id else f"channel:{channel_id or 'unknown'}"
    return f"{platform}:{self_id}:{target}"


def _register_active_request(scope_key: str, request: _ActiveRequest) -> None:
    with _ACTIVE_REQUESTS_LOCK:
        bucket = [item for item in _ACTIVE_REQUESTS.get(scope_key, []) if not item.task.done()]
        bucket.append(request)
        _ACTIVE_REQUESTS[scope_key] = bucket


def _unregister_active_request(scope_key: str, request_id: str) -> None:
    with _ACTIVE_REQUESTS_LOCK:
        bucket = [
            item
            for item in _ACTIVE_REQUESTS.get(scope_key, [])
            if item.request_id != request_id and not item.task.done()
        ]
        if bucket:
            _ACTIVE_REQUESTS[scope_key] = bucket
        else:
            _ACTIVE_REQUESTS.pop(scope_key, None)


def _stop_latest_active_request(scope_key: str) -> bool:
    with _ACTIVE_REQUESTS_LOCK:
        bucket = [item for item in _ACTIVE_REQUESTS.get(scope_key, []) if not item.task.done()]
        if not bucket:
            _ACTIVE_REQUESTS.pop(scope_key, None)
            return False
        _ACTIVE_REQUESTS[scope_key] = bucket
        active = bucket[-1]

    active.stop_event.set()
    active.task.cancel()
    return True


def _sanitize_filename(value: str) -> str:
    safe = re.sub(r'[\\/:*?"<>|\r\n\t]+', "_", str(value or "").strip())
    safe = re.sub(r"\s+", "_", safe).strip("._")
    return (safe or "query")[:48]


def _extract_markdown_images(answer: str) -> tuple[list[str], str]:
    text = str(answer or "")
    image_sources: list[str] = []
    for raw in _MD_IMAGE_RE.findall(text):
        src = str(raw or "").strip().strip("<>").strip()
        if not src:
            continue
        image_sources.append(src)
    cleaned = _MD_IMAGE_RE.sub("", text)
    cleaned = re.sub(r"\n{3,}", "\n\n", cleaned).strip()
    return image_sources, cleaned


def _render_article_skeleton_xml(match: re.Match[str]) -> str:
    block = str(match.group(0) or "")
    title_match = re.search(r"<title\b[^>]*>(.*?)</title>", block, flags=re.IGNORECASE | re.DOTALL)
    title = re.sub(r"\s+", " ", str(title_match.group(1) or "").strip()) if title_match else "未命名骨架"
    lines = [f"文章骨架\n{title}"]
    section_matches = list(re.finditer(r"<section\b([^>]*)>(.*?)</section>", block, flags=re.IGNORECASE | re.DOTALL))
    if section_matches:
        for section_match in section_matches:
            attrs = _as_dict({})
            for key, dq, sq in re.findall(r'([:\w-]+)\s*=\s*(?:"([^"]*)"|\'([^\']*)\')', str(section_match.group(1) or "")):
                attrs[str(key or "").strip().lower()] = str((dq if dq != "" else sq) or "").strip()
            section_name = str(attrs.get("name") or attrs.get("title") or "").strip() or "未命名章节"
            lines.append(f"- {section_name}")
            for claim_match in re.finditer(r"^\s*\[(\d+)\]\s+(.+?)\s*$", str(section_match.group(2) or ""), flags=re.MULTILINE):
                lines.append(f"  [{claim_match.group(1)}] {claim_match.group(2).strip()}")
    else:
        for claim_match in re.finditer(r"^\s*\[(\d+)\]\s+(.+?)\s*$", block, flags=re.MULTILINE):
            lines.append(f"- [{claim_match.group(1)}] {claim_match.group(2).strip()}")
    return "\n".join(lines).strip()


def _render_search_rewrite_xml(match: re.Match[str]) -> str:
    block = str(match.group(0) or "")
    terms = [
        re.sub(r"\s+", " ", str(term_match.group(1) or "").strip())
        for term_match in re.finditer(r"<term\b[^>]*>(.*?)</term>", block, flags=re.IGNORECASE | re.DOTALL)
    ]
    if not terms:
        inner = re.sub(r"</?search_rewrite\b[^>]*>", "", block, flags=re.IGNORECASE)
        for line in inner.splitlines():
            candidate = re.sub(r"^\s*(?:[-*]|\d+[.)])\s*", "", str(line or "").strip())
            candidate = re.sub(r"\s+", " ", candidate).strip()
            if candidate:
                terms.append(candidate)
    if not terms:
        return ""
    return "搜索词重绘\n" + "\n".join(f"- {term}" for term in terms)


def _clean_answer(text: str) -> str:
    cleaned = re.sub(
        r"<summary>\s*(.*?)\s*</summary>",
        lambda m: "\n".join(
            f"> {line}"
            for line in [row.strip() for row in m.group(1).splitlines()]
            if line
        ),
        str(text or ""),
        flags=re.IGNORECASE | re.DOTALL,
    )

    def _image_to_link(match: re.Match[str]) -> str:
        alt = str(match.group(1) or "").strip()
        raw_target = str(match.group(2) or "").strip()
        if not raw_target:
            return match.group(0)
        label = alt or "图片"
        if raw_target.lower().startswith("data:image"):
            return f"{label}: [inline image omitted]"
        return f"{label}: [{raw_target}]({raw_target})"

    cleaned = _MD_IMAGE_RE.sub(_image_to_link, cleaned)
    cleaned = _TASK_LIST_BLOCK_RE.sub("", cleaned)
    cleaned = _SEARCH_REWRITE_BLOCK_RE.sub(_render_search_rewrite_xml, cleaned)
    cleaned = _ARTICLE_SKELETON_BLOCK_RE.sub(_render_article_skeleton_xml, cleaned)
    cleaned = _TOOL_BLOCK_RE.sub("", cleaned)
    cleaned = _TOOL_SELF_CLOSING_RE.sub("", cleaned)
    cleaned = re.sub(
        r"</?(?:search|wiki|sub_agent|page|context_keep|plan_update|context_update|context_delete|tool_results|result|article_skeleton|search_rewrite|section|title|term)\b[^>]*>",
        "",
        cleaned,
    )
    return cleaned.strip()


def _format_tool_argument(name: str, arguments: Any) -> str:
    return format_tool_view_argument(name, _as_dict(arguments), max_items=12, max_chars=160)


def _format_tool_trace_line(name: str, arguments: Any) -> str:
    payload = _as_dict(arguments)
    line = f"> {format_tool_view_text(name, payload, max_chars=200)}"

    extras: list[str] = []
    count = payload.get("_count")
    ok = payload.get("_ok")
    jina_tokens = payload.get("_jina_tokens")
    elapsed_s = payload.get("_elapsed_s")
    if ok is False:
        extras.append("!")
    elif count not in (None, ""):
        extras.append(f"{count} ✓")
    elif ok is True:
        extras.append("✓")
    if jina_tokens not in (None, ""):
        extras.append(f"{jina_tokens}tok")
    if elapsed_s not in (None, ""):
        try:
            extras.append(f"{float(elapsed_s):.1f}s")
        except Exception:
            pass

    if extras:
        line += " " + " ".join(extras)
    return line.strip()


def _render_result_to_src(payload: dict[str, Any]) -> str:
    base64_data = str(payload.get("base64") or "").strip()
    if not base64_data:
        return ""
    mime_type = str(payload.get("mime_type") or "image/png").strip() or "image/png"
    return f"data:{mime_type};base64,{base64_data}"


def _build_notice_chain(text: str, quote_id: str | None = None) -> MessageChain:
    chain = MessageChain(Text(text))
    if quote_id:
        chain = MessageChain(Quote(quote_id)) + chain
    return chain


def _tool_text_line(name: str, args: dict[str, Any]) -> str:
    return f"> {format_tool_view_text(name, args, max_chars=200)}"


def _planned_tool_block(tools: list[tuple[str, dict[str, Any]]] | None) -> str:
    if not tools:
        return ""
    rows: list[str] = []
    for name, args in tools:
        line = _tool_text_line(name, args)
        if line:
            rows.append(line)
    return "\n".join(rows).strip()


def _flatten_provider_tokens(raw: Any) -> list[str]:
    if raw is None:
        return []
    if isinstance(raw, str):
        text = str(raw or "").strip()
        if not text:
            return []
        if "," in text:
            return [part.strip() for part in text.split(",") if str(part or "").strip()]
        return [text]
    if isinstance(raw, (list, tuple, set)):
        items: list[str] = []
        for item in raw:
            items.extend(_flatten_provider_tokens(item))
        return items
    return []


def _provider_summary(config: dict[str, Any], capability: str) -> str:
    candidates = [
        cfg_get(config, f"tools.use.{capability}"),
        cfg_get(config, f"tools.{capability}.providers"),
        cfg_get(config, f"tools.{capability}.provider"),
    ]
    providers: list[str] = []
    seen: set[str] = set()
    for candidate in candidates:
        for item in _flatten_provider_tokens(candidate):
            normalized = normalize_tool_provider_name(str(item or "").strip())
            if not normalized or normalized in seen:
                continue
            seen.add(normalized)
            providers.append(normalized)
    if not providers:
        default = str(DEFAULT_TOOL_SELECTIONS.get(capability, "") or "").strip()
        if default:
            providers.append(default)
            seen.add(default)
    return " / ".join(providers).strip()


def _planned_notice_line(name: str, args: dict[str, Any], config: dict[str, Any]) -> str:
    del config
    return format_tool_view_text(str(name or "").strip(), _as_dict(args), max_chars=200)


def _planned_sub_agent_block(
    tools: list[tuple[str, dict[str, Any]]] | None,
    *,
    config: dict[str, Any],
) -> str:
    if not tools:
        return ""
    rows: list[str] = []
    seen: set[str] = set()
    for name, args in tools:
        line = _planned_notice_line(name, args, config)
        if not line or line in seen:
            continue
        seen.add(line)
        rows.append(line)
    return "\n".join(rows).strip()


def _compose_round_notice(
    thinking: str,
    tools: list[tuple[str, dict[str, Any]]] | None,
    *,
    config: dict[str, Any],
) -> str:
    parts: list[str] = []
    clean_thinking = _clean_answer(thinking)
    if clean_thinking:
        parts.append(clean_thinking)
    tool_block = _planned_sub_agent_block(tools, config=config) if tools else ""
    if tool_block:
        parts.append(tool_block)
    return "\n\n".join(part for part in parts if part).strip()


def _build_entari_render_config(config: dict[str, Any]) -> dict[str, Any]:
    merged = deepcopy(config) if isinstance(config, dict) else {}

    tools = merged.get("tools")
    if not isinstance(tools, dict):
        tools = {}
    else:
        tools = deepcopy(tools)
    merged["tools"] = tools

    tool_index = tools.get("index")
    if not isinstance(tool_index, dict):
        tool_index = {}
    else:
        tool_index = deepcopy(tool_index)
    tools["index"] = tool_index

    md2png_provider = tool_index.get(_ENTARI_RENDER_PROVIDER)
    if not isinstance(md2png_provider, dict):
        md2png_provider = {}
    else:
        md2png_provider = deepcopy(md2png_provider)
    md2png_provider.setdefault("render", "md2png_lite.provider:render_md2png_lite_result")
    tool_index[_ENTARI_RENDER_PROVIDER] = md2png_provider

    md2png_config = merged.get("md2png_lite")
    if not isinstance(md2png_config, dict):
        md2png_config = {}
    else:
        md2png_config = deepcopy(md2png_config)
    md2png_config["theme"] = str(conf.render_theme or md2png_config.get("theme") or "paper").strip() or "paper"
    md2png_config["font_pack"] = "auto"
    merged["md2png_lite"] = md2png_config
    return merged


def _entari_render_font_status(config: dict[str, Any]) -> str:
    try:
        from md2png_lite.font_sync import resolved_font_pack, synced_font_dirs
    except Exception as exc:
        return f"Entari render fonts: md2png-lite font sync unavailable ({exc})."

    effective = str(resolved_font_pack("auto") or "system").strip().lower() or "system"
    font_dirs = [str(item).strip() for item in synced_font_dirs("auto") if str(item).strip()]

    if effective == "noto" and font_dirs:
        return f"Entari render fonts: effective=noto, cache={font_dirs[0]}"
    return "Entari render fonts: effective=system, using system fonts. Install `hyw[notosans]` to enable Noto sync."


async def _build_answer_chain(
    answer_text: str,
    *,
    config: dict[str, Any],
    quote_id: str | None = None,
) -> tuple[MessageChain, MessageChain, bool]:
    image_sources, cleaned_text = _extract_markdown_images(answer_text)
    render_text = _clean_answer(cleaned_text if image_sources else answer_text)

    fallback_chain = _build_notice_chain("渲染出错", quote_id)

    primary_chain = fallback_chain
    rendered_ok = False
    if render_text:
        headless = bool(config.get("headless") not in (False, "false", "0", 0))
        try:
            payload = await render_markdown_result(
                render_text,
                title="",
                config=_build_entari_render_config(config),
                provider=_ENTARI_RENDER_PROVIDER,
                headless=headless,
            )
            rendered_src = _render_result_to_src(payload if isinstance(payload, dict) else {})
            if rendered_src:
                primary_chain = MessageChain(Image(src=rendered_src))
                rendered_ok = True
        except Exception as exc:
            logger.warning("Markdown render with {} failed in entari plugin: {}", _ENTARI_RENDER_PROVIDER, exc)
            try:
                payload = await render_markdown_result(
                    render_text,
                    title="",
                    config=config,
                    headless=headless,
                )
                rendered_src = _render_result_to_src(payload if isinstance(payload, dict) else {})
                if rendered_src:
                    primary_chain = MessageChain(Image(src=rendered_src))
                    rendered_ok = True
            except Exception as fallback_exc:
                logger.warning("Markdown render fallback failed in entari plugin: {}", fallback_exc)
    if quote_id and rendered_ok:
        primary_chain = MessageChain(Quote(quote_id)) + primary_chain

    return primary_chain, fallback_chain, rendered_ok


# ── 图片处理 (复用原有逻辑) ──────────────────────────────────
try:
    from entari_plugin_hyw.entari_misc import process_images
except ImportError:
    # fallback: 内联简单版
    import base64
    import io
    import httpx
    from PIL import Image as PILImage

    def _compress_b64(b64: str, quality: int = 85) -> str:
        img = PILImage.open(io.BytesIO(base64.b64decode(b64)))
        if img.mode in ("RGBA", "P"):
            img = img.convert("RGB")
        buf = io.BytesIO()
        img.save(buf, format="JPEG", quality=quality, optimize=True)
        return base64.b64encode(buf.getvalue()).decode()

    async def process_images(mc: MessageChain, vision_model=None):
        if vision_model == "off":
            return [], None
        images = []
        if mc.get(Image):
            urls = mc[Image].map(lambda item: item.src)
            async with httpx.AsyncClient(timeout=30.0) as client:
                for url in urls:
                    try:
                        resp = await client.get(url)
                        if resp.status_code == 200:
                            b64 = base64.b64encode(resp.content).decode()
                            images.append(_compress_b64(b64))
                    except Exception as e:
                        logger.warning("download image failed: {}", e)
        return images, None


# ── 命令处理 ──────────────────────────────────────────────────

@listen(CommandReceive)
async def remove_at(content: MessageChain):
    return content.lstrip(At)


alc_q = Alconna(conf.question_command, Args["content;?", AllParam])
alc_stop = Alconna(conf.stop_command)


@command.on(alc_stop)
async def handle_stop(session: Session[MessageCreatedEvent]):
    scope_key = _session_request_scope(session)
    quote_id = session.event.message.id if conf.quote else None
    notice = "已停止你最近一次提问。" if _stop_latest_active_request(scope_key) else "没有正在处理的最近一次提问。"
    await session.send(_build_notice_chain(notice, quote_id))


@command.on(alc_q)
async def handle_question(session: Session[MessageCreatedEvent], result: Arparma):
    content = result.all_matched_args.get("content")
    mc = MessageChain(content) if content else MessageChain()

    if session.reply:
        try:
            mc.extend(MessageChain(" ") + session.reply.origin.message)
        except Exception:
            try:
                mc.extend(session.reply.origin.message)
            except Exception:
                logger.debug("Reply message append skipped.")

    user_input = str(mc.get(Text)).strip() if mc.get(Text) else ""
    user_input = _IMG_TAG_RE.sub("", user_input).strip()

    if not user_input and not mc.get(Image):
        return

    quote_id = session.event.message.id if conf.quote else None
    stop_event = threading.Event()
    current_task = asyncio.current_task()
    if current_task is None:
        logger.warning("Failed to register entari request: current task missing.")
        return
    scope_key = _session_request_scope(session)
    request_id = f"{datetime.utcnow().strftime('%Y%m%d%H%M%S%f')}-{id(current_task)}"
    _register_active_request(
        scope_key,
        _ActiveRequest(
            request_id=request_id,
            task=current_task,
            stop_event=stop_event,
        ),
    )

    def _build_round_chain(text: str) -> MessageChain:
        chain = MessageChain(Text(text))
        if quote_id:
            chain = MessageChain(Quote(quote_id)) + chain
        return chain

    async def _send_round_notice(text: str) -> None:
        if stop_event.is_set():
            return
        await session.send(_build_round_chain(text))

    try:
        if not conf.verbose:
            try:
                await _send_round_notice("Thinking...")
            except Exception as exc:
                logger.warning("Thinking notice send failed in entari plugin: {}", exc)

        images, _ = await process_images(mc, None)
        question = user_input or "请根据图片内容进行分析并回答。"
        loop = asyncio.get_running_loop()
        sent_round_messages: set[str] = set()
        sent_round_lock = threading.Lock()
        config = load_config()
        stats = Stats()

        def _on_rewind(thinking: str, tools: list[tuple[str, dict[str, Any]]] | None = None) -> None:
            if not conf.verbose or stop_event.is_set():
                return
            clean = _compose_round_notice(thinking, tools, config=config)
            if not clean:
                return
            with sent_round_lock:
                if clean in sent_round_messages:
                    return
                sent_round_messages.add(clean)
            try:
                future = asyncio.run_coroutine_threadsafe(
                    _send_round_notice(clean),
                    loop,
                )
                future.result()
            except Exception as exc:
                logger.warning("Round message send failed in entari plugin: {}", exc)

        runner = run_stream if config.get("stream") not in (False, "false", "0", 0) else run
        kwargs: dict[str, Any] = {
            "config": config,
            "stats": stats,
            "images": images,
            "stop_checker": stop_event.is_set,
        }
        if runner is run_stream and conf.verbose:
            kwargs["on_rewind"] = _on_rewind
        final_content = await asyncio.to_thread(
            runner,
            question,
            **kwargs,
        )

        if stop_event.is_set():
            return
        answer_text = str(final_content or "").strip()
        if not answer_text:
            return

        primary_chain, fallback_chain, rendered_ok = await _build_answer_chain(
            answer_text,
            config=config,
            quote_id=quote_id,
        )
        try:
            if stop_event.is_set():
                return
            await session.send(primary_chain)
        except Exception as exc:
            if rendered_ok:
                logger.warning("Rendered message send failed in entari plugin: {}", exc)
                if not stop_event.is_set():
                    await session.send(fallback_chain)
            else:
                raise

    except asyncio.CancelledError:
        stop_event.set()
        return
    except StopRequestedError:
        stop_event.set()
        return
    except Exception as exc:
        logger.exception("Error in hyw execution: {}", exc)
        return
    finally:
        _unregister_active_request(scope_key, request_id)


# ── 生命周期 ──────────────────────────────────────────────────

@listen(Startup)
async def on_startup():
    logger.info("HYW plugin startup: loading configuration...")

    try:
        cfg = load_config()
        logger.success(
            "HYW plugin ready. model={}, api_base={}, headless={}",
            cfg.get("model"),
            cfg.get("api_base"),
            cfg.get("headless"),
        )
        logger.info(_entari_render_font_status(cfg))
        headless = cfg.get("headless") not in (False, "false", "0", 0)
        try:
            logger.info("tools startup (headless={})", headless)
            startup_tools(headless=headless, config=cfg)
            logger.info("tools startup done")
        except Exception as exc:
            logger.warning("tools startup skipped: {}", exc)
    except Exception as exc:
        logger.warning("HYW plugin ready with unknown config state: {}", exc)


@listen(Cleanup)
async def cleanup_resources():
    logger.info("Cleaning up hyw resources (tools)...")
    shutdown_tools()


__plugin__ = metadata(
    "hyw",
    author=[{"name": "kumo", "email": "dev@example.com"}],
    version=__version__,
    config=PluginConfig,
)



from __future__ import annotations

from .config import build_model_config, cfg_get, get_model_profiles, load_config
from .render import (
    RenderCallable,
    RenderResult,
    render_markdown_base64,
    render_markdown_result,
)
from .web_search import WebToolSuite, on_shutdown, on_startup, page_extract, wait_until_ready, web_search

__all__ = [
    "RenderCallable",
    "RenderResult",
    "WebToolSuite",
    "build_model_config",
    "cfg_get",
    "get_model_profiles",
    "load_config",
    "on_shutdown",
    "on_startup",
    "page_extract",
    "render_markdown_base64",
    "render_markdown_result",
    "wait_until_ready",
    "web_search",
]
